@@ -6,7 +6,7 @@
 ;; URL: https://github.com/xuchunyang/eshell-git-prompt
 ;; Package-Requires: ((emacs "24.1") (cl-lib "0.5") (dash "2.11.0"))
 ;; Keywords: eshell git
-;; Version: 0.1.2
+;; Version: 0.1.3
 ;; Created: 09/11/2015
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -73,6 +73,12 @@
     (powerline
      eshell-git-prompt-powerline
      eshell-git-prompt-powerline-regexp)
+    (multiline
+     eshell-git-prompt-multiline
+     eshell-git-prompt-multiline-regexp)
+    (multiline2
+     eshell-git-prompt-multiline2
+     eshell-git-prompt-multiline2-regexp)
     ;; Only a single $
     (simple
      eshell-git-prompt-simple
@@ -86,6 +92,60 @@ You can add your own theme to this list, then run
   :group 'eshell-prompt
   :type '(repeat (list symbol symbol symbol)))
 
+(defface eshell-git-prompt-exit-success-face
+  '((((class color) (background light)) :foreground "forest green")
+    (((class color) (background  dark)) :foreground "green"))
+  "Face for the success char"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-exit-fail-face
+  '((((class color) (background light)) :foreground "red")
+    (((class color) (background  dark)) :foreground "red"))
+  "Face for the fail char"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-directory-face
+  '((((class color) (background light)) :foreground "steel blue")
+    (((class color) (background  dark)) :foreground "cyan"))
+  "Face for the directory"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-modified-face
+  '((((class color) (background light)) :foreground "dark orange")
+    (((class color) (background  dark)) :foreground "red"))
+  "Face for the git 'modified' char"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-add-face
+  '((((class color) (background light)) :foreground "dim gray")
+    (((class color) (background  dark)) :foreground "white"))
+  "Face for the git 'add' char"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-branch-face
+  '((((class color) (background light)) :foreground "dim gray")
+    (((class color) (background  dark)) :foreground "dark gray"))
+  "Face for the git branch"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-robyrussell-git-face
+  '((((class color) (background light)) :foreground "blue")
+    (((class color) (background  dark)) :foreground "blue"))
+  "Face for the git indicator"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-robyrussell-branch-face
+  '((((class color) (background light)) :foreground "red")
+    (((class color) (background  dark)) :foreground "red"))
+  "Face for the git branch"
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-robyrussell-git-dirty-face
+  '((((class color) (background light)) :foreground "dark orange")
+    (((class color) (background  dark)) :foreground "yellow"))
+  "Face for the git dirty"
+  :group 'eshell-faces)
+
 (defface eshell-git-prompt-powerline-dir-face
   '((t :background "steel blue"))
   "Face for directory name in eshell git prompt theme `powerline`"
@@ -101,6 +161,59 @@ You can add your own theme to this list, then run
   "Face for git branch (not clean) in eshell git prompt theme `powerline`"
   :group 'eshell-faces)
 
+(defface eshell-git-prompt-multiline-secondary-face
+  '((((class color) (background light)) :foreground "light gray")
+    (((class color) (background  dark)) :foreground "dim gray"))
+  "Face for secondary part in eshell git prompt theme `multiline`. e.g. separator, horizontal line, date."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline-command-face
+  '((((class color) (background light)) :foreground "slate blue")
+    (((class color) (background  dark)) :foreground "gold"))
+  "Face for command user typed in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline-sign-face
+  '((t :foreground "deep pink"))
+  "Face for prompt sign in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-secondary-face
+  '((t (:foreground "#51afef" :weight ultra-bold)))
+  "Face for secondary part in eshell git prompt theme `multiline`. e.g. separator, horizontal line, date."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-usr-face
+  '((t (:foreground "magenta" :weight ultra-bold)))
+  "Face for prompt sign in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-dir-face
+  '((t (:foreground "white" :weight ultra-bold)))
+  "Face for prompt sign in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-host-face
+  '((t (:foreground "green" :weight ultra-bold)))
+  "Face for prompt sign in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-git-face
+  '((t (:foreground "red" :weight ultra-bold)))
+  "Face for prompt sign in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-fail-face
+  '((t (:foreground "red" :weight ultra-bold)))
+  "Face for prompt sign in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
+(defface eshell-git-prompt-multiline2-command-face
+  '((((class color) (background light)) (:foreground "slate blue" :weight ultra-bold))
+    (((class color) (background  dark)) (:foreground "gold" :weight ultra-bold)))
+  "Face for command user typed in eshell git prompt theme `multiline`."
+  :group 'eshell-faces)
+
 
 ;;; * Internal
 
@@ -109,6 +222,15 @@ You can add your own theme to this list, then run
   (declare (indent 1))
   `(propertize ,str 'face (list ,@properties)))
 
+(defun eshell-git-prompt---str-read-only (str)
+  "Make STR read-only and return the string."
+  (add-text-properties 0 (length str)
+		       '(read-only t
+		         front-sticky (read-only)
+		         rear-nonsticky (read-only))
+		       str)
+  str)
+
 (defmacro eshell-git-prompt-str-append (str suffix)
   "Append STR with SUFFIX and set the value of symbol STR to the result."
   (if (symbolp str)
@@ -259,14 +381,16 @@ If working directory is clean, return nil."
 ;;; * Themes
 
 (defun eshell-git-prompt-simple ()
-  (if (= (user-uid) 0) "# " "$ "))
+  (eshell-git-prompt---str-read-only
+   (if (= (user-uid) 0) "# " "$ ")))
 
 (defconst eshell-git-prompt-simple-regexp "^[#$] ")
 
 ;; the Eshell default prompt
 (defun eshell-git-prompt-default-func ()
-  (concat (abbreviate-file-name (eshell/pwd))
-          (if (= (user-uid) 0) " # " " $ ")))
+  (eshell-git-prompt---str-read-only
+   (concat (abbreviate-file-name (eshell/pwd))
+           (if (= (user-uid) 0) " # " " $ "))))
 
 (defconst eshell-git-prompt-default-regexp "^[^#$\n]* [#$] ")
 
@@ -282,13 +406,12 @@ It looks like:
   (let (beg dir git-branch git-dirty end)
     ;; Beg: start symbol
     (setq beg
-          (with-face "➜"
-            :foreground (if (eshell-git-prompt-exit-success-p)
-                            "green" "red")))
+          (with-face "➜" (if (eshell-git-prompt-exit-success-p)
+                            'eshell-git-prompt-exit-success-face 'eshell-git-prompt-exit-fail-face)))
 
     ;; Dir: current working directory
     (setq dir (with-face (eshell-git-prompt--shorten-directory-name)
-                :foreground "cyan"))
+                'eshell-git-prompt-directory-face))
 
     ;; Git: branch/detached head, dirty status
     (when (eshell-git-prompt--git-root-dir)
@@ -296,21 +419,22 @@ It looks like:
 
       (setq git-branch
             (concat
-             (with-face "git:(" :foreground "blue")
-             (with-face (eshell-git-prompt--readable-branch-name) :foreground "red")
-             (with-face ")" :foreground "blue")))
+             (with-face "git:(" 'eshell-git-prompt-robyrussell-git-face)
+             (with-face (eshell-git-prompt--readable-branch-name) 'eshell-git-prompt-robyrussell-branch-face)
+             (with-face ")" 'eshell-git-prompt-robyrussell-git-face)))
 
       (setq git-dirty
             (when (eshell-git-prompt--collect-status)
-              (with-face "✗" :foreground "yellow"))))
+              (with-face "✗" 'eshell-git-prompt-robyrussell-git-dirty-face))))
 
     ;; End: To make it possible to let `eshell-prompt-regexp' to match the full prompt
     (setq end (propertize "$" 'invisible t))
 
     ;; Build prompt
-    (concat (mapconcat #'identity (-non-nil (list beg dir git-branch git-dirty)) " ")
-            end
-            " ")))
+    (eshell-git-prompt---str-read-only
+     (concat (mapconcat #'identity (-non-nil (list beg dir git-branch git-dirty)) " ")
+             end
+             " "))))
 
 (defconst eshell-git-prompt-robbyrussell-regexp "^[^$\n]*\\\$ ")
 
@@ -319,99 +443,103 @@ It looks like:
 
 (defun eshell-git-prompt-git-radar ()
   "Eshell Git prompt inspired by git-radar."
-  (concat
-   (with-face "➜"
-     :foreground (if (eshell-git-prompt-exit-success-p)
-                     "green" "red"))
-   " "
-   (with-face (eshell-git-prompt--shorten-directory-name)
-     :foreground "cyan")
-   ;; Yo, we are in a Git repo, display some information about it
-   (when (eshell-git-prompt--git-root-dir)
-     (setq eshell-git-prompt-branch-name
-           (eshell-git-prompt--branch-name)
-           eshell-git-prompt-remote-branch-name
-           (eshell-git-prompt--remote-branch-name))
-     (concat
-      " "
-      (with-face "git:(" :foreground "dark gray")
+  (eshell-git-prompt---str-read-only
+   (concat
+    (with-face "➜"
+      (if (eshell-git-prompt-exit-success-p)
+          'eshell-git-prompt-exit-success-face 'eshell-git-prompt-exit-fail-face))
+    " "
+    (with-face (eshell-git-prompt--shorten-directory-name)
+      'eshell-git-prompt-directory-face)
+    ;; Yo, we are in a Git repo, display some information about it
+    (when (eshell-git-prompt--git-root-dir)
+      (setq eshell-git-prompt-branch-name
+            (eshell-git-prompt--branch-name)
+            eshell-git-prompt-remote-branch-name
+            (eshell-git-prompt--remote-branch-name))
+      (concat
+       " "
+       (with-face "git:(" 'eshell-git-prompt-branch-face)
 
-      ;; Branch name
-      (with-face (eshell-git-prompt--readable-branch-name)
-        :foreground "gray")
+       ;; Branch name
+       (with-face (eshell-git-prompt--readable-branch-name)
+         'eshell-git-prompt-branch-face)
 
-      ;; Local commits
-      (when eshell-git-prompt-remote-branch-name
-        (let ((local-behind (eshell-git-prompt--commits-behind-of-remote))
-              (local-ahead (eshell-git-prompt--commits-ahead-of-remote)))
-          (cond ((and (> local-ahead 0) (> local-behind 0))
-                 (concat " "
-                         (number-to-string local-behind)
-                         (with-face "⇵" :foreground "yellow")
-                         (number-to-string local-ahead)))
-                ((> local-behind 0)
-                 (concat " "
-                         (number-to-string local-behind)
-                         (with-face "↓" :foreground "red")))
-                ((> local-ahead 0)
-                 (concat " "
-                         (number-to-string local-ahead)
-                         (with-face "↑" :foreground "LimeGreen"))))))
+       ;; Local commits
+       (when eshell-git-prompt-remote-branch-name
+         (let ((local-behind (eshell-git-prompt--commits-behind-of-remote))
+               (local-ahead (eshell-git-prompt--commits-ahead-of-remote)))
+           (cond ((and (> local-ahead 0) (> local-behind 0))
+                  (concat " "
+                          (number-to-string local-behind)
+                          (with-face "⇵" :foreground "yellow")
+                          (number-to-string local-ahead)))
+                 ((> local-behind 0)
+                  (concat " "
+                          (number-to-string local-behind)
+                          (with-face "↓" :foreground "red")))
+                 ((> local-ahead 0)
+                  (concat " "
+                          (number-to-string local-ahead)
+                          (with-face "↑" :foreground "LimeGreen"))))))
 
-      (with-face ")" :foreground "dark gray")
+       (with-face ")" 'eshell-git-prompt-branch-face)
 
-      ;; File status
-      (-when-let (git-status (eshell-git-prompt--collect-status))
-        (-let [(&plist :untracked untracked
-                       :new-added new-added
-                       :modified-updated modified-updated
-                       :modified modified)
-               git-status]
-          (concat
-           ;; Updated to index changes
-           (let (group1)
-             (setq group1
-                   (concat
-                    (when (> new-added 0)
-                      (concat
-                       (number-to-string new-added)
-                       (with-face "A" :foreground "green")))
-                    (when (> modified-updated 0)
-                      (concat
-                       (number-to-string modified-updated)
-                       (with-face "M" :foreground "green")))))
-             (when (> (length group1) 0)
-               (concat " " group1)))
-           ;; Modified but not updated
-           (when (> modified 0)
-             (concat " " (number-to-string modified)
-                     (with-face "M" :foreground "red")))
-           ;; Untracked file
-           (when (> untracked 0)
-             (concat " " (number-to-string untracked)
-                     (with-face "A" :foreground "white"))))))))
-   ;; To make it possible to let `eshell-prompt-regexp' to match the full prompt
-   (propertize "$" 'invisible t) " "))
+       ;; File status
+       (-when-let (git-status (eshell-git-prompt--collect-status))
+         (-let [(&plist :untracked untracked
+                        :new-added new-added
+                        :modified-updated modified-updated
+                        :modified modified)
+                git-status]
+           (concat
+            ;; Updated to index changes
+            (let (group1)
+              (setq group1
+                    (concat
+                     (when (> new-added 0)
+                       (concat
+                        (number-to-string new-added)
+                        (with-face "A" 'eshell-git-prompt-add-face)))
+                     (when (> modified-updated 0)
+                       (concat
+                        (number-to-string modified-updated)
+                        (with-face "M" 'eshell-git-prompt-modified-face)))))
+              (when (> (length group1) 0)
+                (concat " " group1)))
+            ;; Modified but not updated
+            (when (> modified 0)
+              (concat " " (number-to-string modified)
+                      (with-face "M" 'eshell-git-prompt-modified-face)))
+            ;; Untracked file
+            (when (> untracked 0)
+              (concat " " (number-to-string untracked)
+                      (with-face "A" 'eshell-git-prompt-add-face))))))))
+    ;; To make it possible to let `eshell-prompt-regexp' to match the full prompt
+    (propertize "$" 'invisible t) " ")))
 
 (defconst eshell-git-prompt-git-radar-regexp "^[^$\n]*\\\$ ")
 
 
 ;; Powerline
 
+;; For https://github.com/xuchunyang/eshell-git-prompt/issues/8
+(defun eshell-git-prompt-powerline-dir ()
+  (abbreviate-file-name (eshell/pwd)))
+
 (defun eshell-git-prompt-powerline ()
   (let ((segment-separator "\xe0b0")
-        (plusminus         "\x00b1")
         (branch            "\xe0a0")
         (detached          "\x27a6")
         (cross             "\x2718")
-        dir git git-bg)
+        dir git git-face)
     (setq dir
           (propertize
            (concat
             " "
             (unless (eshell-git-prompt-exit-success-p)
               (concat cross " "))
-            (abbreviate-file-name (eshell/pwd))
+            (eshell-git-prompt-powerline-dir)
             " ")
            'face 'eshell-git-prompt-powerline-dir-face))
     (setq git
@@ -422,28 +550,99 @@ It looks like:
                     'eshell-git-prompt-powerline-clean-face))
             (setq eshell-git-prompt-branch-name (eshell-git-prompt--branch-name))
             (propertize
-                (concat " "
-                        (-if-let (branch-name eshell-git-prompt-branch-name)
-                            (concat branch " " branch-name)
-                          (concat detached " "(eshell-git-prompt--commit-short-sha)))
-                        " ")
-              'face git-face)))
-    (concat
-     (if git
-         (concat dir
-                 (with-face segment-separator
-                   :foreground (face-background 'eshell-git-prompt-powerline-dir-face)
-                   :background (face-background git-face))
-                 git
-                 (with-face segment-separator
-                   :foreground (face-background git-face)))
-       (concat dir
-               (with-face segment-separator
-                 :foreground (face-background 'eshell-git-prompt-powerline-dir-face))))
-     (propertize "$" 'invisible t) " ")))
+             (concat " "
+                     (-if-let (branch-name eshell-git-prompt-branch-name)
+                         (concat branch " " branch-name)
+                       (concat detached " "(eshell-git-prompt--commit-short-sha)))
+                     " ")
+             'face git-face)))
+    (eshell-git-prompt---str-read-only
+     (concat
+      (if git
+          (concat dir
+                  (with-face segment-separator
+                    :foreground (face-background 'eshell-git-prompt-powerline-dir-face)
+                    :background (face-background git-face))
+                  git
+                  (with-face segment-separator
+                    :foreground (face-background git-face)))
+        (concat dir
+                (with-face segment-separator
+                  :foreground (face-background 'eshell-git-prompt-powerline-dir-face))))
+      (propertize "$" 'invisible t) " "))))
 
 (defconst eshell-git-prompt-powerline-regexp "^[^$\n]*\\\$ ")
 
+(defun eshell-git-prompt-multiline ()
+  "Eshell Git prompt inspired by spaceship-prompt."
+  (let (separator hr dir git git-dirty time sign command)
+    (setq separator (with-face " | " 'eshell-git-prompt-multiline-secondary-face))
+    (setq hr (with-face (concat "\n" (make-string (/ (window-total-width) 2) ?─) "\n") 'eshell-git-prompt-multiline-secondary-face))
+    (setq dir
+          (concat
+           (with-face "🗀" 'eshell-git-prompt-directory-face)
+           (concat  (abbreviate-file-name (eshell/pwd)))))
+    (setq git
+          (concat (with-face "⎇" 'eshell-git-prompt-exit-success-face)
+                  (concat (eshell-git-prompt--branch-name))))
+    (setq git-dirty
+          (when (eshell-git-prompt--branch-name)
+            (if (eshell-git-prompt--collect-status)
+                (with-face " ✎" 'eshell-git-prompt-modified-face)
+              (with-face " ✔" 'eshell-git-prompt-exit-success-face))))
+    (setq time (with-face (format-time-string "%I:%M:%S %p") 'eshell-git-prompt-multiline-secondary-face))
+    (setq sign
+          (if (= (user-uid) 0)
+              (with-face "\n#" 'eshell-git-prompt-multiline-sign-face)
+            (with-face "\nλ" 'eshell-git-prompt-multiline-sign-face)))
+    (setq command (with-face " " 'eshell-git-prompt-multiline-command-face))
+
+    ;; Build prompt
+    (eshell-git-prompt---str-read-only
+     (concat hr dir separator git git-dirty separator time sign command))))
+
+(defconst eshell-git-prompt-multiline-regexp "^[^$\n]*λ ")
+
+(defun eshell-git-prompt-multiline2 ()
+  "Eshell Git prompt inspired by spaceship-prompt."
+  (let (beg separator usr at host dir git time sign command)
+    (setq beg (with-face "\n┌─(" 'eshell-git-prompt-multiline2-secondary-face))
+    (setq separator (with-face ")──(" 'eshell-git-prompt-multiline2-secondary-face))
+    (setq usr (with-face (user-login-name) ''eshell-git-prompt-multiline2-usr-face))
+    (setq at (with-face "@" 'eshell-git-prompt-multiline2-secondary-face))
+    (setq host (with-face (system-name) 'eshell-git-prompt-multiline2-host-face))
+    (setq dir
+          (with-face (eshell-git-prompt--shorten-directory-name)
+	    'eshell-git-prompt-multiline2-dir-face))
+    (setq git
+	  (concat 
+	   (if (eshell-git-prompt--branch-name)
+	       (concat
+		separator
+		(with-face "⎇ " 'eshell-git-prompt-exit-success-face)
+		(with-face (eshell-git-prompt--branch-name)
+		  'eshell-git-prompt-multiline2-git-face))
+	     "")
+	   (when (eshell-git-prompt--branch-name)
+	     (if (eshell-git-prompt--collect-status)
+		 (with-face " ✗" 'eshell-git-prompt-modified-face)
+	       (with-face " ✔" 'eshell-git-prompt-exit-success-face)))))
+    (setq time (with-face (format-time-string "%I:%M:%S %p")
+		 'eshell-git-prompt-multiline2-command-face))
+    (setq sign
+	  (concat
+	   (with-face ")\n└─" 'eshell-git-prompt-multiline2-secondary-face)
+	   (if (not (eshell-git-prompt-exit-success-p))
+	       (with-face ">>" 'eshell-git-prompt-multiline2-fail-face)
+             (with-face ">>" 'eshell-git-prompt-multiline2-secondary-face))))
+    (setq command (with-face " " 'eshell-git-prompt-multiline2-command-face))
+
+    ;; Build prompt
+    (eshell-git-prompt---str-read-only
+     (concat beg usr at host separator dir git separator time sign command))))
+
+(defconst eshell-git-prompt-multiline2-regexp "^[^$\n]*└─>> ")
+
 (defvar eshell-git-prompt-current-theme nil)
 
 ;;;###autoload
